[[_TOC_]]

# Virtual Domain for Email Architecture

## Introduction

This document is based on the [original ikiwki page][1] (also archived
[here](../archive/virtual-domain.mdwn)).

The Virtual Domain for Email service provides Stanford departments and
business groups with their own vanity email domain forwarding. Department
and group administrators manage the email aliases for their domains via a
web interface. For example, `adamhl@math.stanford.edu` can be used instead of
`adamhl@stanford.edu`.

One motivation for this service is historical: in the past departments ran
their own mail servers and used their own domains in the e-mail
address. As an incentive to get departments to use the centralized email
service this service allows the members of that department to continue
using their old email addresses with the department domain. This avoided
the hassle of telling everyone of the changed e-mail address.

Another motiviation is that a virtual domain gives departments and groups
"branded" e-mail addresses as well as an expanded e-mail address
namespace: `admin@law.stanford.edu`, `student-help@math.stanford.edu`,
etc.

## Design

Departments and groups will be able to request (via HelpSU) a virtual
email domain to be hosted on the campus email gateways (Proofpoint and smtp
servers). Emails to aliases at the virtual domain will have the same
virus and junk mail filtering as emails to the `@stanford.edu` domain.
Virtual domain emails will also be subject to the same secure email
routing and encryption rules as emails to `@stanford.edu`.

A virtual domain can also have secondary domains attached to it. Emails
to secondary domains will be treated and routed the same as emails to
the primary domain.

  For example, to request a virtual domain idg.stanford.edu, the group
  administrator should submit a HelpSU ticket with the following data:

    * Domain: idg.stanford.edu
    * Secondary domains: infrastructure-delivery-group.stanford.edu,
                         itservices-idg.stanford.edu
    * Management workgroup: itservices:idg
    * Contact email: its-idg-team@lists.stanford.edu
    * Oracle Financial Account: xxx-xxx-xxx

  In the above example, emails to foo@itservices-idg.stanford.edu will be
  treated the same as foo@idg.stanford.edu.

  Any changes to the this metadata will have to be requested via the
  HelpSU interface.  The metadata will be stored in a MySQL database.

  __CAVEAT__: The Oracle Financial Account information is mainly there to
  weed out frivolous requests from non-business groups and users since
  there is likely no charge for this service.

  __PHASE 2__: This will become a web request form outside of HelpSU which
  will automatically generate some of the configuration fragments.  This
  entire process will also be somewhat manual but we can streamline
  certain parts.


## Glossary

* _container_: this is the email virtual domain that is used in e-mail
addresses. Examples: `idg.stanford.edu`, `math.stanford.edu`.

* _sub-domains_: a subdomain of a container.

* _alias_: A container contains a list of e-mail aliases.  An alias
contains one or more _forwarding addresses_.  An alias is used when
sending e-mail. E-mail sent to this address would be forwarded to the
aliases forwarding addresses.

* _mailbox_: a synonym of _alias_.

* _forwarding address_: These are e-mail addresses that mail sent to an
_alias_ are delivered to.

**Example:** The container `idg` has three aliases with the following
forwarding addresses:
```
adamhl:
  - adamhl@stanford.edu
  - adamh456@lewenberg.com
whm:
  - whm@stanford.edu
team:
  - adamhl@stanford.edu
  - whm@stanford.edu
  - rra@stanford.edu
```
Mail sent to `team@idg.stanford.edu` will be forwarded to
`adamhl@stanford.edu`, `whm@stanford.edu`, and `rra@stanford.edu`.

## Mail Flow

Let's assume that `test.stanford.edu` is an email virtual domain and that
one of its aliases is `jdoe` where `jdoe` forwards its mail to
`jdoe1@stanford.edu` and `jroe@gmail.com`.

CASE 1: MX record

1. A user sends an e-mail to "jdoe@test.stanford.edu".

1. The MX record for `test.stanford.edu` resolves to Proofpoint servers.

1. The Proofpoint servers load the container and alias information from
LDAP every 30 minutes. A Proofpoint mail server receives the e-mail and,
using this LDAP information it has loaded, looks up the aliases and
sees that the aliases are `jdoe1@stanford.edu` and `jroe@gmail.com`.

1. The Proofpoint servers delivers the mail to `jdoe1@stanford.edu` and
`jroe@gmail.com`.

CASE 2: SMTP server

1. A user sends an e-mail to "jdoe@test.stanford.edu".

1. An SMTP server (e.g., `smtp4.stanford.edu`) receives the e-mail and,
having been configured to look up the domain in LDAP, looks up the alias
jdoe@test.stanford.edu and asks the Proofpoint servers to deliver copies
to `jdoe1@stanford.edu` and `jroe@gmail.com`.

1. The Proofpoint servers delivers the copies to their recipients.


**Question**: Why do we need the SMTP servers to be able to handle virtual
email domains since the mail is being sent to Proofpoint anyway? Jason
Cowart says this is useful for a couple of reasons. The first is that mail
originating from the mailman servers do not need to go through Proofpoint
a second time. The second is in case we stop using Proofpoint.

**Question**: Why not have the virtual domain MX record resolve to
smtp.stanford.edu and remove the code in Proofpoint?

The Proofpoint configuration and code to support this was written and is
maintained Proofpoint Professional services. New containers need to be
submitted to Proofpoint.


## Data Structure

* A container HAS ONE OR MORE aliases.

* An alias HAS ONE OR MORE forwarding addresses.

* In the database schema below, an alias's forwarding addresses are stored as
a comma-delimited string of e-mail addresses.

* The database values are synced to the values in LDAP and *not*
vice-versa: The information in the database is the **source of truth**.

### Database Schema

#### `containers`

```
+------------+-------------------------+------+-----+---------+-------+
| Field      | Type                    | Null | Key | Default | Extra |
+------------+-------------------------+------+-----+---------+-------+
| container  | varchar(128)            | NO   | PRI | NULL    |       |
| workgroup  | varchar(256)            | NO   |     | NULL    |       |
| contact    | varchar(256)            | NO   |     | NULL    |       |
| finance    | varchar(64)             | NO   |     | NULL    |       |
| status     | enum('active','frozen') | NO   |     | NULL    |       |
| dns_domain | varchar(255)            | YES  |     | NULL    |       |
+------------+-------------------------+------+-----+---------+-------+
```

#### `aliases`

```
+-------------+---------------+------+-----+---------+-------+
| Field       | Type          | Null | Key | Default | Extra |
+-------------+---------------+------+-----+---------+-------+
| mail        | varchar(256)  | NO   | PRI | NULL    |       |
| maildrop    | varchar(512)  | NO   |     | NULL    |       |
| container   | varchar(128)  | NO   | PRI | NULL    |       |
| owner       | varchar(8)    | YES  |     | NULL    |       |
| description | varchar(1024) | YES  |     | NULL    |       |
+-------------+---------------+------+-----+---------+-------+
```

Each entry in `aliases` corresponds to an entry in the LDAP of the form
`cn=<mail>,cn=<container>,cn=applications,dc=stanford,dc=edu`. The `maildrop` field
is a comma-delimited list of e-mail addresses that are expanded into the LDAP
multi-valued attribute `suMailDrop`.

Example: the record in `aliases`
```
+---------+------------------------------------------------+-----------+---------+-----------------------------------------------+
| mail    | maildrop                                       | container | owner   | description                                   |
+---------+------------------------------------------------+-----------+---------+-----------------------------------------------+
| buddy3  | frankie4@lewenberg.com,frankie44@lewenberg.com | test      | adamhl  | asdasdasd                                     |
+---------+------------------------------------------------+-----------+---------+-----------------------------------------------+
```
corresponds to the LDAP entry
```
dn: cn=buddy3,cn=test,cn=applications,dc=stanford,dc=edu
objectClass: suMailRoute
suAdministrator: adamhl
cn: buddy3
mail: buddy3
description: asdasdasd
suMailDrop: frankie4@lewenberg.com
suMailDrop: frankie44@lewenberg.com
```

#### `domains`

```
+-----------+--------------+------+-----+---------+-------+
| Field     | Type         | Null | Key | Default | Extra |
+-----------+--------------+------+-----+---------+-------+
| container | varchar(256) | NO   | PRI |         |       |
| domain    | varchar(256) | NO   | PRI |         |       |
+-----------+--------------+------+-----+---------+-------+
```



[1]: https://ikiwiki.stanford.edu/service/email/virtual-domain/
